@@ -1,27 +1,34 @@
 from ray.rllib.env.base_env import BaseEnv
-from ray.rllib.env.dm_env_wrapper import DMEnv
-from ray.rllib.env.dm_control_wrapper import DMCEnv
-from ray.rllib.env.unity3d_env import Unity3DEnv
-from ray.rllib.env.pettingzoo_env import PettingZooEnv
-from ray.rllib.env.multi_agent_env import MultiAgentEnv
+from ray.rllib.env.env_context import EnvContext
 from ray.rllib.env.external_env import ExternalEnv
 from ray.rllib.env.external_multi_agent_env import ExternalMultiAgentEnv
-from ray.rllib.env.vector_env import VectorEnv
-from ray.rllib.env.env_context import EnvContext
+from ray.rllib.env.multi_agent_env import MultiAgentEnv
 from ray.rllib.env.policy_client import PolicyClient
 from ray.rllib.env.policy_server_input import PolicyServerInput
+from ray.rllib.env.remote_vector_env import RemoteVectorEnv
+from ray.rllib.env.vector_env import VectorEnv
+
+from ray.rllib.env.wrappers.dm_env_wrapper import DMEnv
+from ray.rllib.env.wrappers.dm_control_wrapper import DMCEnv
+from ray.rllib.env.wrappers.group_agents_wrapper import GroupAgentsWrapper
+from ray.rllib.env.wrappers.pettingzoo_env import PettingZooEnv
+from ray.rllib.env.wrappers.pettingzoo_env import ParallelPettingZooEnv
+from ray.rllib.env.wrappers.unity3d_env import Unity3DEnv
 
 __all__ = [
     "BaseEnv",
-    "MultiAgentEnv",
-    "ExternalEnv",
-    "ExternalMultiAgentEnv",
-    "VectorEnv",
-    "EnvContext",
     "DMEnv",
     "DMCEnv",
-    "Unity3DEnv",
+    "EnvContext",
+    "ExternalEnv",
+    "ExternalMultiAgentEnv",
+    "GroupAgentsWrapper",
+    "MultiAgentEnv",
     "PettingZooEnv",
+    "ParallelPettingZooEnv",
     "PolicyClient",
     "PolicyServerInput",
+    "RemoteVectorEnv",
+    "Unity3DEnv",
+    "VectorEnv",
 ]
